# coding: utf-8
#
# Copyright (c) 2004 Brian Sutherland
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
TTML parse/write library.

This library implements extensions to the python minidom classes which make it
easier to parse/write TTML documents.

This library only implements a sub-set of TTML, or more specifically the short
cut/common functions as implementing a full MathML parser was not necessary. A
schema is provided with the parser which can validate inputs to the parser. The
aim of this schema is not to completely describe TTML, but only to provide a
means of parser input checking.

The TTML format is taken, as accurately as possible, from the paper:
    E. zcan, Towards an XML based standard for Timetabling Problems: TTML,
    http://cse.yeditepe.edu.tr/~eozcan/research/TTML/

Please note that this paper is not and does not try to be a standard or
complete.

WARNING: A large proportion of these interfaces are unimplemented and subject
change.
"""

from sets import Set, ImmutableSet
import datetime

from zope.interface import implements

from minittml import interfaces

class DocInfo:

    implements(interfaces.IDocInfo, interfaces.ICreateDocInfo)

    def __init__(self, attrs):
        self.attrs = attrs

    def getDocumentAttrs(self):
        return self.attrs

    def __eq__(self, other):
        if not interfaces.IDocInfo.providedBy(other):
            return False
        if self.attrs != other.attrs:
            return False
        return True

class Description:
    """A simple data object for descriptive data about the problem or solutions.

    authors is a list of srings representing authors.
    desc is the decription and is a sting.
    refs is a list of stings representing the references.
    """

    implements(interfaces.IDescription,
            interfaces.ICreateDescription)

    def __init__(self, authors, desc, refs):
        self.authors = authors
        self.desc = desc
        self.refs = refs

    def getAuthors(self):
        return self.authors

    def getDesc(self):
        return self.desc

    def getRefs(self):
        return self.refs

    def __eq__(self, other):
        if not interfaces.IDescription.providedBy(other):
            return False
        if self.getAuthors() != other.getAuthors() or\
                self.getDesc() != other.getDesc() or\
                self.getRefs() != other.getRefs():
            return False
        return True


class InputDescription(Description):
    """A sub class for the input data which implements the required interfaces.
    """

    implements(interfaces.IInputDescription)

    def __eq__(self, other):
        if not interfaces.IInputDescription.providedBy(other):
            return False
        if not Description.__eq__(self, other):
            return False
        return True


class TTMLSet:

    def __init__(self, name, base_set_members):
        self.name = name
        self.members = base_set_members

    def getBaseSetName(self):
        return self.name

    def getBaseSetMembers(self):
        return self.members

    def __eq__(self, other):
        if self.getBaseSetMembers() != other.getBaseSetMembers() or\
                self.getBaseSetName() != other.getBaseSetName():
            return False
        return True


class AttributedSet(TTMLSet):
    """A simple data object for attributed sets.

    Data is a list of 2 member tuples. The first member is a string or
    attributed set and the second is a dictionary of attribute=values.

    All attribute names and values will be converted to strings.

    Recursive attributed sets are only usefull to build classifiers where
    attributes do not make sense.
    """

    def __init__(self, name, base_set_members, attribute_names=(),
            attributes=()):
        """Initialize."""
        TTMLSet.__init__(self, name, base_set_members)
        self.attribute_names = attribute_names
        self.attributes = attributes

    def __eq__(self, other):
        if not TTMLSet.__eq__(self, other):
            return False
        if self.getAttributeNames() != other.getAttributeNames():
            return False
        for member in self.getBaseSetMembers():
            for attr in self.getAttributeNames():
                if self.getAttribute(attr, member) != \
                        other.getAttribute(attr, member):
                    return False
        return True

    def getAttributeNames(self):
        return self.attribute_names

    def getAttribute(self, attribute_set_name, base_set_member):
        for i in range(len(self.members)):
            if self.members[i] == base_set_member:
                for j in range(len(self.attribute_names)):
                    if self.attribute_names[j] == attribute_set_name:
                        return self.attributes[j][i]
        raise KeyError


class VariableSet(AttributedSet):

    implements(interfaces.IVariableSet,
            interfaces.ICreateVariableSet)

    def __init__(self, name, base_set_members, attribute_names=(),
            attributes=(), element_attr_names=(), element_attrs=()):
        AttributedSet.__init__(self, name, base_set_members,
                attribute_names=attribute_names, attributes=attributes)
        self.element_attr_names = element_attr_names
        self.element_attrs = element_attrs

    def __eq__(self, other):
        if not interfaces.IVariableSet.providedBy(other):
            return False
        if not AttributedSet.__eq__(self, other):
            return False
        if self.getElementAttributeNames() != \
                other.getElementAttributeNames():
            return False
        for member in self.getBaseSetMembers():
            for el_attr in self.getElementAttributeNames():
                if self.getElementAttribute(el_attr, member) != \
                        other.getElementAttribute(el_attr, member):
                    return False
        return True

    def getElementAttributeNames(self):
        return self.element_attr_names

    def getElementAttribute(self, attr_name, base_set_member):
        for i in range(len(self.members)):
            if self.members[i] == base_set_member:
                for j in range(len(self.element_attr_names)):
                    if self.element_attr_names[j] == attr_name:
                        return self.element_attrs[j][i]
        raise KeyError


class Domain(AttributedSet):

    implements(interfaces.IDomain,
            interfaces.ICreateDomain)

    def __eq__(self, other):
        if not interfaces.IDomain.providedBy(other):
            return False
        if not AttributedSet.__eq__(self, other):
            return False
        return True



class Classifier(TTMLSet):

    implements(interfaces.IClassifier,
            interfaces.ICreateClassifier)

    def __eq__(self, other):
        if not interfaces.IClassifier.providedBy(other):
            return False
        if not TTMLSet.__eq__(self, other):
            return False
        return True


class TimeDomain:
    """A simple time domain object describing the time domain.

    For a description of the arguments, please look at the interfaces.
    """

    implements(interfaces.ITimeDomain,
            interfaces.ICreateTimeDomain)

    def __init__(self, name, intervals, isdiscrete):
        if not isinstance(name, (str, unicode)):
            raise TypeError("Name must be a string, not %s" % type(name))
        for i in intervals:
            if not len(i) == 2 or not isinstance(i[0], datetime.datetime)\
                    or not isinstance(i[1], datetime.timedelta):
                raise TypeError("intervals must be tuples of (datetime,"
                        " timedelta)")
        if not isinstance(isdiscrete, bool):
            raise TypeError("isdiscrete must be a Boolean, not %s"
                % isdiscrete.__class__)
        self.name = name
        self.intervals = intervals
        self.isdiscrete = isdiscrete

    def getName(self):
        return self.name

    def isDiscrete(self):
        return self.isdiscrete

    def getIntervals(self):
        return self.intervals

    def __eq__(self, other):
        if not interfaces.ITimeDomain.providedBy(other):
            return False
        if self.getName() != other.getName() or\
                self.isDiscrete() != other.isDiscrete() or\
                self.getIntervals() != other.getIntervals():
            return False
        return True


class Function:

    implements(interfaces.IFunction)

    def __init__(self, arguments, attributes={}, name=None):
        self.name = name
        self.arguments = arguments
        self.attributes = attributes

    def __eq__(self, other):
        if not interfaces.IFunction.providedBy(other):
            return False
        if self.getArguments() != other.getArguments()\
                or self.getAttributes() != other.getAttributes()\
                or self.getName() != other.getName():
            return False
        return True

    def getName(self):
        return self.name

    def getArguments(self):
        return self.arguments

    def getAttributes(self):
        return self.attributes


class TmatrixTimeDomain(Function):
    """A simple time domain object describing the time domain.

    For a description of the arguments, please look at the interfaces.
    """

    implements(interfaces.ITmatrixTimeDomain,
            interfaces.ICreateTimeDomainFunction)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.ITmatrixTimeDomain.providedBy(other):
            return False
        if not Function.__eq__(self, other):
            return False
        return True


class Domainsvar:
    """A data object for a domains variable.

    name = name of the domain
    domains = list of domains in the cartesian product
    """

    implements(interfaces.IDomainsvar,
            interfaces.ICreateDomainsvar)

    def __init__(self, name, domains):
        self.domains = domains
        self.name = name

    def getDomainsvarName(self):
        return self.name

    def getDomains(self):
        return self.domains

    def __eq__(self, other):
        if not interfaces.IDomainsvar.providedBy(other):
            return False
        if self.getDomainsvarName() != other.getDomainsvarName() or\
                self.getDomains() != other.getDomains():
            return False
        return True


class ConstraintFunction(Function):

    implements(interfaces.ICreateConstraintFunction)

    def __init__(self, arguments, attributes={}, name=None, ishard=True):
        Function.__init__(self, arguments, attributes=attributes, name=name)
        if isinstance(ishard, bool):
            self.ishard = ishard
        else:
            raise TypeError("%s must be type" % (ishard, bool))

    def __eq__(self, other):
        if not Function.__eq__(self, other):
            return False
        if self.isHard() != other.isHard():
            return False
        return True


    def isHard(self):
        return self.ishard


class NotSameConstraint(ConstraintFunction):
    """A simple data object for NotSame constraint."""

    implements(interfaces.INotSameConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.INotSameConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class NoOverlapConstraint(ConstraintFunction):
    """A simple data object for NotOverlap constraint."""

    implements(interfaces.INoOverlapConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.INoOverlapConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class PresetConstraint(ConstraintFunction):
    """A simple data object for Preset constraint."""

    implements(interfaces.IPresetConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IPresetConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class ExcludeConstraint(ConstraintFunction):
    """A simple data object for exclude constraint."""

    implements(interfaces.IExcludeConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IExcludeConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class OrderingConstraint(ConstraintFunction):
    """A simple data object for Ordering constraint."""

    implements(interfaces.IOrderingConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IOrderingConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class EventSprConstraint(ConstraintFunction):
    """A simple data object for EventSpr constraint."""

    implements(interfaces.IEventSprConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IEventSprConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class FullSprConstraint(ConstraintFunction):
    """A simple data object for FullSpr constraint."""

    implements(interfaces.IFullSprConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IFullSprConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class FreeSprConstraint(ConstraintFunction):
    """A simple data object for FreeSpr constraint."""

    implements(interfaces.IFreeSprConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IFreeSprConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class AttrCompConstraint(ConstraintFunction):
    """A simple data object for AttrComp constraint."""

    implements(interfaces.IAttrCompConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IAttrCompConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class ResNoClashConstraint(ConstraintFunction):
    """A simple data object for ResNoClash constraint."""

    implements(interfaces.IResNoClashConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IResNoClashConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class ChksumConstraint(ConstraintFunction):
    """A simple data object for Chksum constraint."""

    implements(interfaces.IChksumConstraint)

    # XXX - input type checking

    def __eq__(self, other):
        if not interfaces.IChksumConstraint.providedBy(other):
            return False
        if not ConstraintFunction.__eq__(self, other):
            return False
        return True


class Result:
    """A minimal data object for a TTML result.

    desc = the descriptive data about the result
            (As an object implementing IDescription)
    domains = list of domains in the cartesian product
    """

    implements(interfaces.IResult,
            interfaces.ICreateResult)

    def __init__(self, desc, best):
        # XXX - input checking
        self.desc = desc
        self.best = best

    def getDescription(self):
        return self.desc

    def getBestResult(self):
        return self.best

    def __eq__(self, other):
        if not interfaces.IResult.providedBy(other):
            return False
        if not self.getDescription() == other.getDescription() or\
                self.getBestResult() != other.getBestResult():
            return False
        return True
