# coding: utf-8
#
# Copyright (c) 2004 Brian Sutherland
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Interfaces for writing TTML documents - Development Version

The primary purpose of this module is only to provide an high level object/xml
translation layer, leaving out most error checking. i.e. you can create a
completely invalid TTML document with this. Indeed the format of TTML is quite
complex and requires more validation than XML schemas are able to provide.

These interfaces make it easier to write TTML documents by providing a set of
high level objects corredponding to parts of a TTML document. The actual
reading and writing is performed using a wrapper/extension around the minidom
module. Data is transferred with objects implementing the Data Object
Interfaces. These intefaces are method only, read only so even 3rd party
objects can be easily written to the document.

A set of data object creation iterfaces are also provided that documents the
creation of objects from the minittml.data module.

Perhaps one day it will not be possible to create an invalid TTML doc with this
interface.

The interface is compiled, as accurately as possible, from the paper:
    E. zcan, Towards an XML based standard for Timetabling Problems: TTML,
    http://cse.yeditepe.edu.tr/~eozcan/research/TTML/

Please note that this paper is not and does not try to be a standard or
complete.

WARNING: These interfaces are unstable and subject change without notice.
"""

from zope.interface import Interface, Attribute

#
# Ancestors of Data Object Interfaces
#

#
# These are here just to save space because a lot of data objects end up
# implementing similar interfaces. They also simplify the implementation
# somewhat. You cannot use IDoc.addObject() with them.
#

class IDescription(Interface):
    """This interface is for the descriptive data about a section of a doc.

    This contains the author names, description string and references for the
    input or results section.
    """

    def getAuthors():
        """Returns a list of author names as a tuple of strings."""

    def getDesc():
        """Returns a string containting the description or None."""

    def getRefs():
        """Returns a list of references as a tuple of strings."""


class IFunction(Interface):
    """A common interface for MathML functions."""

    def getName():
        """Returns a string which is the name of the constaint.

        If Null, then the constraint is not named.
        """

    def getArguments():
        """Returns a list of constraint arguments.

        The number and type of the members of the list depend on the function
        being implemented. See the specialized function interfaces derived
        from this interface for details.
        """

    def getAttributes():
        """Return a dictionary of attribute:value pairs of the constraint.

        Returns the attributes of the function, which attributes
        are settable/usefull depends on the individual constraint. See the
        specialized constraint interfaces for more details.
        """


class IConstraintFunction(IFunction):
    """A common interface for constraint functions."""

    def isHard():
        """True or False depending on whether the constraint if hard or soft"""


#
# Data Object Interfaces
#

#
# Any object implementing one (and only one) of the following interfaces can be
# written to the document via IDoc.addObject()
#

class IDocInfo(Interface):
    """Descriptive information of the entire document."""

    def getDocumentAttrs():
        """Returns a dictionary of attribute:string pairs.

        These are the attributes of the top level node in a ttml document. They
        include:

        type: The type of timetabling problem being addressed. Suggested values
                are: 'University-Course-Timetabling'
        lastUpdate: The date and time of the last modification.

        NOTE: the keyword xmlns has no effect
        """

class IInputDescription(IDescription):
    """The descriptive data from the input-data document section."""


class IVariableSet(Interface):
    """An interface for a variables set for the varibles section of a TTML doc.

    A set in TTML can either be an simple set only containing variables
    or numbers. Or it can be an attributed set with a base simple set
    and sets of attributes. Each member of a set of attributes is related to
    a member of the base set. These sets are described by the TTML tag
    <attrset>.

    In addition to this the base set members can have attributes written as XML
    attributes i.e. <ci duration='2'>A100<ci>, these are called element
    attributes.
    """

    def getBaseSetName():
        """Returns the name of the base set"""

    def getBaseSetMembers():
        """Returns a tuple of base set members.

        Yes, it is a set, but the order is important, see the README.
        """

    def getAttributeNames():
        """Returns a tuple of the names of the attribute sets."""

    def getAttribute(attribute_set_name, base_set_member):
        """Return the value of an attribute.

        Returns a KeyError if either the attribute_set_name and/or the
        base_set_member do not exist. If the attribute has no value an empty
        string should be returned.
        """

    def getElementAttributeNames():
        """Returns a tuple of the names of element attributes.

        These attributes are not in a separate set of attributes, but are
        found directly on the node. eg. duration in <ci duration='1'>A100</ci>
        """

    def getElementAttribute(element_attribute_name, base_set_member):
        """Return the value of an attribute.

        Returns a KeyError if either the element_attribute_name and/or the
        base_set_member do not exist.

        A return value of None means that the attribute will not be set.
        If an empty string is returned, the attribute will be set to an empty
        string (e.g. <ci duration=''>)
        """


class IDomain(Interface):
    """An interface for a domain.

    Please see the description in IVariableSet.

    Exception: Domains do not have element attributes.
    """

    def getBaseSetName():
        """See the equivilent method in IVariableSet."""

    def getBaseSetMembers():
        """See the equivilent method in IVariableSet."""

    def getAttributeNames():
        """See the equivilent method in IVariableSet."""

    def getAttribute(attribute_set_name, base_set_member):
        """See the equivilent method in IVariableSet."""


class IDomainsvar(Interface):
    """This reprsent a domainsvar as a cartesian product of domains.

    This object sets up the domainsvar section of a TTML document as a
    cartesian product of multiple domains.
    """

    def getDomainsvarName():
        """Returns the name of the domains variable"""

    def getDomains():
        """Returns a list domain names (normally including the time domain).

        The cartesian product of these domains is set as the timetabling
        problem domain.
        """


class ITimeDomain(Interface):
    """Defines an interface for a basic time domain object.

    This imterface implements the the most general and basic way of
    representing a timetable. As a set of intervals.

    Time domains are special instacnces of sets and are treated specially.
    A timetable is a set of time intervals which are composed of a starting
    time and a duration.

    Different types of time domains must have different information and be able
    to respond to different queries, therefore they are implemeted as separate
    interfaces. Behaviour when an object implementing two of these interfaces
    is passed is undefined.

    Timetables can be labelled as discrete or continuous. But what this actually
    means is unclear.
    """

    def getName():
        """Returns the name string of the time domain"""

    def isDiscrete():
        """Returns True if the time domain is discrete, else False."""

    def getIntervals():
        """Returns a list of intervals in the time domain.

        interval
            An interval is a tuple of (datetime.datetime, datetime.timedelta)
            describing the starting time and duration of the interval.
        """


class ITmatrixTimeDomain(IFunction):
    """A timetable object that uses the tmatrix shortcut function.

    A discrete timetable composed of x rows and y columns. The resulting
    time intervals do not have a starting time or duration, but can be
    thought of as dis-connected time slots.

    Arguments List (returned by getArguments):
    arg1 = the number of rows (not a string!)
    arg2 = the number of columns (not a string!)

    Possible Function Attributes (returned by getAttributes):
        TODO
    """


class IClassifier(Interface):
    """An interface for a classifier.

    Please see the equivilent in IVariableSet.

    Exceptions: Classifiers have no attributes or element attributes, but can
    contain another classifier as a base member.
    """

    def getBaseSetName():
        """See the equivilent method in IVariableSet."""

    def getBaseSetMembers():
        """See the equivilent method in IVariableSet.

        An exception is that the members themselves can be objects implementing
        IClassifier. This allows recursive classifiers.
        """


class INotSameConstraint(IConstraintFunction):
    """"""


class INoOverlapConstraint(IConstraintFunction):
    """"""


class IPresetConstraint(IConstraintFunction):
    """An interface for a preset constraint(O2) from TTML.

    A pre-set constraint limits the domain/s of variable/s to certain values.

    This does not implement fully the preset constraint function, and as such
    can only represent one domain. So either use only one domain (i.e. time)
    or specify a dimension.

    Arguments List (returned by getArguments):
    arg1 = The name of a member of the variables set or of a classifier of
        variables to exclude from the domain. (A string)
    arg2 = The name of a member of the domain for which the constriant applies,
        or a name of a classifier of members of that domain. (A string)

    Possible Function Attributes (returned by getAttributes):
    dim = Name of a domain as a string.
        The dimansion(domain) the constraint applies to as a string. If only
        the time domain exists, then it is superflous. If there is more than
        one domain defined in the domainsvar, then it is necessary as multiple
        domain constaints are unimplemented.
    projection = "self"|"child"|"single"
        The type of projection to be applied to classifiers listed as arguments.
    """


class IExcludeConstraint(IConstraintFunction):
    """An interface for a exclusion constraint(O3) in TTML.

    An exclusion constraint limits the domain/s of variable/s to preclude
    certain values.

    Arguments List (returned by getArguments):
    arg1 = The name of a member of the variables set or of a classifier of
        variables to exclude from the domain. (A string)
    arg2 = The name of a member of the domain for which the constriant applies,
        or a name of a classifier of members of that domain. (A string)

    Possible Function Attributes (returned by getAttributes):
    dim = Name of a domain as a string.
        The dimansion(domain) the constraint applies to as a string. If only
        the time domain exists, then it is superflous. If there is more than
        one domain defined in the domainsvar, then it is necessary as multiple
        domain constaints are unimplemented.
    projection = "self"|"child"|"single"
        The type of projection to be applied to classifiers listed as arguments.
    """


class IOrderingConstraint(IConstraintFunction):
    """"""


class IEventSprConstraint(IConstraintFunction):
    """"""


class IFullSprConstraint(IConstraintFunction):
    """"""


class IFreeSprConstraint(IConstraintFunction):
    """"""


class IAttrCompConstraint(IConstraintFunction):
    """"""


class IResNoClashConstraint(IConstraintFunction):
    """"""


class IChksumConstraint(IConstraintFunction):
    """"""


class IResult(Interface):
    """Interface for an object representing a TTML result."""

    def getDescription():
        """Get an object implementing IDescription"""

    def getBestResult():
        """Returns the best results from this result as a 2 dimensional list.

        the index is result[variable_index][domainsvar_index] = domain_member

        thus the sublists correspond to the variables defined in the variables
        sections, and the elements of the sub lists correspond to the domains
        defined in the domainsvar. The value itself is a member of the domain.

        The order of the outer and inner lists corresponds to the order of the
        variables and domainsvar sections respectively.

        Currently values can only be strings.
        """

#
# Data Object Creation Interfaces
#


# These interfaces are implemented by the objects in minittml.data but need not
# be implemented by objects passed to the document. So it is best to document
# them separately.

class ICreateDocInfo(Interface):
    """Initialize an object with the IDocInfo object."""

    def __init__(attrs):
        """Initialize, attrs is a dictionary described in IDocInfo."""


class ICreateDescription(Interface):
    """Initialization of descriptive information."""

    def __init__(authors, desc, refs):
        """Initiialize descriptive information for input-data/result section.

        authors = a list of strings representing the author names.
        desc = a string of the description for the section.
        refs = a list of strings of references for this section.
        """


class ICreateVariableSet(Interface):
    """Initialization interface for variable sets."""

    def __init__(name, base_set_members,
                attribute_names=(), attributes=((), ),
                element_attr_names=(), element_attrs=((), )):
        """Initialize.

        name - A string as the name of the classifier.
        base_set_members - A tuple of base set members as strings.
        attribute_names - the names of attributes for the sets of attributes
            defined in an attrset TTML emement. A tuple of strings.
        attributes - A tuple of tuples indexed by
            [attribute_name][base_set_member] and containing the attributes as
            strings (MathML <ci> tag) or numbers (MathML <cn> tag).
        element_attr_names - The names of attributes for the members of the
            main set(attribute means standard XML attribute here). The duration
            attribute is a fine example. A tuple of strings/None.
        attributes - A tuple of tuples indexed by
            [element_attr_name][base_set_member] and containing the attributes
            as strings.
        """


class ICreateDomain(Interface):
    """Initialization interface for domains."""

    def __init__(name, base_set_members,
                attribute_names=(), attributes=()):
        """Initialize the domain.

        name - A string as the name of the classifier.
        base_set_members - A tuple of base set members as strings.
        attribute_names - the names of attributes for the sets of attributes
            defined in an attrset TTML emement. A tuple of strings.
        attributes - A tuple of tuples indexed by
            [attribute_name][base_set_member] and containing the attributes as
            strings (MathML <ci> tag) or numbers (MathML <cn> tag).
        """


class ICreateDomainsvar(Interface):
    """Initilazation of the domainsvar."""

    def __init__(name, domains):
        """Initialize the domainsvar as a cartesian product of domains.

        name - The name of the domainsvar declaration.
        domains - list/tuple of strings of domain names (Be carefull here,
            order is important)
        """


class ICreateTimeDomain(Interface):
    """Initialization interfaces for a basic TimeDomain."""

    def __init__(name, intervals, isdiscrete):
        """Initialize the time domain.

        name = a string which is the name of the time domain.
        intervals = a list of (datetime.datetime, datetimg.timedelta) tuples
            which form the elements of the time domian.
        isdiscrete = boolean (True if the time domain is discrete).
        """


class ICreateTimeDomainFunction(Interface):
    """Initialization interfaces for a TimeDomain as a function."""

    def __init__(ishard, arguments, attributes={}, name=None):
        """Initialize the constraint.

        name - the name of the function as a string(optional)
        arguments - a list of the arguments, returned by
            getArguments. The type and number are different for every
            time domain function (See the individual time domain interfaces).
        attributes - A dictionary of the function attributes, returned by
            getAttributes. The type and number are different for every
            time domain function (See the individual time domain interfaces).
        """


class ICreateClassifier(Interface):
    """Initialization interface for classifiers."""

    def __init__(name, base_set_members):
        """Initialize the classifier.

        name - A string as the name of the classifier.
        base_set_members - A tuple of base set members as strings or objects
            implementing IClassifier.
        """


class ICreateConstraintFunction(Interface):
    """Initialization of a constraint as a constraint function."""

    def __init__(arguments, attributes={}, name=None, ishard=True):
        """Initialize the constraint.

        ishard - True if this is a hard constraint, else False (default True).
        name - the name of the constraint function as a string(optional)
        arguments - a list of the constraint arguments, returned by
            getArguments. The type and number are different for every
            constraint (See the individual constraint interfaces).
        attributes - A dictionary of the function attributes, returned by
            getAttributes. The type and number are different for every
            constraint(see the individual constraint interfaces for details).
        """

class ICreateResult(Interface):
    """Intialize a result data object"""

    def __init__(desc, best):
        """Create the object.

        desc = an object implementig IDescriptive.
        best = TODO
        """

#
# Document Node Interfaces
#

class IDoc(Interface):
    """An high level interface for reading and writing a TTML Doc."""

    doc = Attribute("""The DOM document tree using the minidom module with
            ttmldom extensions""")

    def addObject(obj):
        """Write an object implementing a data object interfaces to the doc.

        This just does the right thing, make sure your objects implement only
        one of the data object interfaces and throw them in here. Even makes
        sure to overwrite things that should be overwritten.
        """

    def popInputDescription():
        """Return the descriptive info ftom the input-data section.

        As an object implementing IInputDescription.
        """

    def popVariableSet():
        """Return the variable as an object implementing IVariableSet."""

    def popTimeDomain():
        """Return the time domain as an object.

        implements:
            ITimeDomain
            ITimeDomainAsTmatrix
        """

    def popDomain():
        """Pop a domain as an object implementing IDomain."""

    def popDomainsvar():
        """Pop the domainsvar object which implements IDomainsvar."""

    def popRootClassifier():
        """Pop a root classifier object implementing IClassifier."""

    def popHardConstraint():
        """Pop a hard constraint as an object.

        The type of constraint returned is dependent on the interface returned.
        Currently only these constraint types are understood by the parser:
            IPresetConstraint (O2)
            IExcludeConstraint (O3)

        The document should be validated by the schema included with the parser
        to make sure that the parser will understand the constraints presented.

        This parser will almost certainly never be able to read arbitary
        constraints defined in MathML.
        """

    def popSoftConstraint():
        """Same as the hard constraint equivalent."""

    def popResult():
        """Pop a result object implementing IResult."""


class IDocumentFactory(Interface):
    """Interface for a factory object to easily parse/create documents."""

    def parse(file):
        """Parse a file.

        file is either a file object or a string which is the name of a file.
        """

    def create():
        """Return an empty document helper object implementing IDoc."""
