#
# Copyright (c) 2004 Brian Sutherland
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import unittest
from sets import Set, ImmutableSet
from datetime import timedelta
from datetime import datetime

from zope.interface.verify import verifyObject

from minittml import interfaces
from minittml import data
from minittml import ttmldom

class TestDocInfo(unittest.TestCase):
    """Test the DocInfo object."""

    def test(self):
        attrs = {'xmlns': 'http://schema', 'type': 'Type A'}
        obj = data.DocInfo(attrs)
        verifyObject(interfaces.ICreateDocInfo, obj)
        verifyObject(interfaces.IDocInfo, obj)
        self.assertEqual(obj.getDocumentAttrs(), attrs)

    def testEquality(self):
        obj1 = data.DocInfo({'xmlns': 'http://schema', 'type': 'Type A'})
        obj2 = data.DocInfo({'xmlns': 'http://schema', 'type': 'Type A'})
        self.assertEqual(obj1, obj2)
        obj3 = data.DocInfo({'xmlns': 'http://schema2', 'type': 'Type B'})
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestDescription(unittest.TestCase):
    """Test the Description data object."""

    def test(self):
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        desc = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        verifyObject(interfaces.ICreateDescription, desc)
        verifyObject(interfaces.IDescription, desc)
        self.assertEqual(desc.getAuthors(), descriptive_authors)
        self.assertEqual(desc.getDesc(), descriptive_desc)
        self.assertEqual(desc.getRefs(),  descriptive_refs)

    def testEquality(self):
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        obj = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        descriptive_authors = ['C. Bukowski', u'P.J. Gutierez']
        descriptive_desc = u'The same minds.'
        descriptive_refs = ['Ham on Rye', u'Anclado en la tierra de nadie']
        changedobj = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        self.assertEqual(changedobj, obj)
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'xxx', 'Anclado en la tierra de nadie']
        changedobj = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        self.assertNotEqual(changedobj, obj)
        descriptive_authors = ['C. Bukowski', 'xxx']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        changedobj = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        self.assertNotEqual(changedobj, obj)
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'xxx'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        changedobj = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        self.assertNotEqual(changedobj, obj)
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie',
                'xxx']
        changedobj = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        self.assertNotEqual(changedobj, obj)
        self.assertNotEqual(obj, 's')


class TestInputDescription(unittest.TestCase):
    """A sub class for the input data which implements the required interfaces.
    """

    def test(self):
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        desc = data.InputDescription(descriptive_authors, descriptive_desc,
                descriptive_refs)
        verifyObject(interfaces.IInputDescription, desc)
        changedobj = data.InputDescription(descriptive_authors,
                descriptive_desc + 'c', descriptive_refs)
        self.assertNotEqual(changedobj, desc)
        changedobj = 'ddd'
        self.assertNotEqual(changedobj, desc)


class TestTTMLSet(unittest.TestCase):
    """Test the TTMLSet object"""

    def test(self):
        attrset_name = 'The Set'
        members = ('Form A - English', 'Form A - Science', 'Form B - English')
        attrset = data.TTMLSet(attrset_name, members)
        self.assertEqual(attrset.getBaseSetName(), attrset_name)
        self.assertEqual(attrset.getBaseSetMembers(), members)

    def testEquality(self):
        """Tested in VariablesSet."""
        pass


class TestAttributedSet(unittest.TestCase):
    """Test the VariablesSet data object."""

    def test(self):
        attrset_name = 'The Set'
        members = ('Form A - English', 'Form A - Science', 'Form B - English')
        attr_names = ('noOfStudents', )
        attrs = (('55', '22', '' ), )
        attrset = data.AttributedSet(attrset_name, members,
                attribute_names=attr_names, attributes=attrs)
        self.assertEqual(attrset.getAttributeNames(), attr_names)
        for i in [0, 1, 2]:
            member = members[i]
            self.assertEqual(
                    attrset.getAttribute(attr_names[0], member),
                    attrs[0][i])
        self.assertRaises(KeyError, attrset.getAttribute, 'noOfStudents',
                'Form C')
        self.assertRaises(KeyError, attrset.getAttribute, 'teacher',
                'Form A - Science')

    def testEquality(self):
        """Tested in VariablesSet."""
        pass


class TestVariableSet(unittest.TestCase):

    def test(self):
        attrset_name = 'The Set'
        members = ('Form A - English', 'Form A - Science', 'Form B - English')
        element_attr_names = ('duration', )
        element_attrs = (('55', '', None,), )
        attrset = data.VariableSet(attrset_name, members,
                element_attr_names=element_attr_names,
                element_attrs=element_attrs)
        verifyObject(interfaces.IVariableSet, attrset)
        verifyObject(interfaces.ICreateVariableSet, attrset)
        self.assertEqual(attrset.getElementAttributeNames(), element_attr_names)
        for i in [0, 1, 2]:
            member = members[i]
            self.assertEqual(
                    attrset.getElementAttribute(element_attr_names[0], member),
                    element_attrs[0][i])
        self.assertRaises(KeyError, attrset.getElementAttribute,
                'duration', 'Form C')
        self.assertRaises(KeyError, attrset.getElementAttribute,
                'teacher', 'Form A - Science')

    def testEquality(self):
        factory = ttmldom.DocumentFactory()
        doc = factory.create()
        set_name = 'The Set'
        members = ('Form A - English', 'Form A - Science', 'Form B - English')
        attr_names = ('noOfStudents', )
        attrs = (('55', '22', ''), )
        element_attr_names = ('duration', )
        element_attrs = (('55', '', None), )
        set = data.VariableSet(set_name, members,
                attribute_names=attr_names, attributes=attrs,
                element_attr_names=element_attr_names,
                element_attrs=element_attrs)
        # Now change the set in small ways and make sure it results in not =
        changedsets = [ 'string' ]
        # Change the order of the data
        changedmembers = ('Form A - English', 'Form B - English',
                'Form A - Science')
        changedattrs = ( ('55', '', '22'), )
        changedelement_attrs = (('55', None, ''), )
        changedset = data.VariableSet(set_name, changedmembers,
                attribute_names=attr_names, attributes=changedattrs,
                element_attr_names=element_attr_names,
                element_attrs=changedelement_attrs)
        changedsets.append(changedset)
        #set name
        changedset = data.VariableSet(set_name + 'c', members,
                attribute_names=attr_names, attributes=attrs,
                element_attr_names=element_attr_names,
                element_attrs=element_attrs)
        changedsets.append(changedset)
        #different attr value
        changedattrs = (('55', '1000000', ''), )
        changedset = data.VariableSet(set_name, members,
                attribute_names=attr_names, attributes=changedattrs,
                element_attr_names=element_attr_names,
                element_attrs=element_attrs)
        changedsets.append(changedset)
        #different no of attrs
        changedattr_names = ('noOfStudents', 'whodingy')
        changedattrs = (('55', '22', ''), ('45', '', ''))
        changedset = data.VariableSet(set_name, members,
                attribute_names=changedattr_names, attributes=changedattrs,
                element_attr_names=element_attr_names,
                element_attrs=element_attrs)
        changedsets.append(changedset)
        #different base membername
        changedmembers = ('Form A - English', 'Form A - Science',
                'xxxxxxxxxxx')
        changedset = data.VariableSet(set_name, changedmembers,
                attribute_names=attr_names, attributes=attrs,
                element_attr_names=element_attr_names,
                element_attrs=element_attrs)
        changedsets.append(changedset)
        #extra base member
        changedmembers = ('Form A - English', 'Form A - Science',
                'Form B - English', 'hghhg')
        changedattrs = (('55', '22', '', ''), )
        changedelement_attrs = (('55', '', None, ''), )
        changedset = data.VariableSet(set_name, changedmembers,
                attribute_names=attr_names, attributes=changedattrs,
                element_attr_names=element_attr_names,
                element_attrs=changedelement_attrs)
        #different element attr name
        changedelement_attr_names = ('xxxxxx', )
        changedset = data.VariableSet(set_name, members,
                attribute_names=attr_names, attributes=attrs,
                element_attr_names=changedelement_attr_names,
                element_attrs=element_attrs)
        changedsets.append(changedset)
        #different element attr value
        changedelement_attrs = (('54', '', None, ), )
        changedset = data.VariableSet(set_name, members,
                attribute_names=attr_names, attributes=attrs,
                element_attr_names=element_attr_names,
                element_attrs=changedelement_attrs)
        changedsets.append(changedset)
        #Now Test!
        for changedset in changedsets:
            self.assertNotEqual(changedset, set)


class TestDomain(unittest.TestCase):

    def test(self):
        attrset_name = 'The Set'
        members = ('Form A - English', 'Form A - Science', 'Form B - English')
        attr_names = ('class', )
        attrs = (('55', '50', '55'), )
        attrset = data.Domain(attrset_name, members,
                attribute_names=attr_names, attributes=attrs)
        verifyObject(interfaces.IDomain, attrset)
        verifyObject(interfaces.ICreateDomain, attrset)

    def testEquality(self):
        attrset_name = 'The Set'
        members = ('Form A - English', 'Form A - Science', 'Form B - English')
        attr_names = ('class', )
        attrs = (('55', '50', '55'), )
        attrset = data.Domain(attrset_name, members,
                attribute_names=attr_names, attributes=attrs)
        attrset2 = data.Domain(attrset_name, members,
                attribute_names=attr_names, attributes=attrs)
        attrset3 = data.Domain(attrset_name, members)
        self.assertEquals(attrset, attrset2)
        self.assertNotEquals(attrset, attrset3)
        self.assertNotEquals(attrset, 's')


class TestTimeDomain(unittest.TestCase):
    """Test the TimeDomain data object."""

    def test(self):
        """Test the vanilla time domain data object."""
        intervals = [
                (datetime(1984,4,13,hour=8), timedelta(minutes=50)),
                (datetime(1984,4,13,hour=9), timedelta(minutes=50)),
                (datetime(1984,4,14,hour=8), timedelta(minutes=50)),
                (datetime(1984,4,14,hour=9), timedelta(minutes=50))]
        vanilla = data.TimeDomain(u'Time', intervals, False)
        vanilla = data.TimeDomain('Time', intervals, False)
        verifyObject(interfaces.ITimeDomain, vanilla)
        verifyObject(interfaces.ICreateTimeDomain, vanilla)
        self.assertEqual(vanilla.getName(), 'Time')
        self.assertEqual(vanilla.getIntervals(), intervals)
        self.assertEqual(vanilla.isDiscrete(), False)
        self.assertRaises(TypeError, data.TimeDomain, 5, intervals, False)
        self.assertRaises(TypeError, data.TimeDomain, 'Time', intervals, '1')
        intervals.append((5, 6))
        self.assertRaises(TypeError, data.TimeDomain, 'Time', intervals, False)

    def testEquality(self):
        intervals = [
                (datetime(1984,4,13,hour=8), timedelta(minutes=50)),
                (datetime(1984,4,13,hour=9), timedelta(minutes=50)),
                (datetime(1984,4,14,hour=8), timedelta(minutes=50)),
                (datetime(1984,4,14,hour=9), timedelta(minutes=50))]
        obj = data.TimeDomain('Time', intervals, False)
        changedobj = data.TimeDomain('Time', intervals, False)
        self.assertEqual(changedobj, obj)
        changedobj = data.TimeDomain('Tme', intervals, False)
        self.assertNotEqual(changedobj, obj)
        changedobj = data.TimeDomain('Time', intervals, True)
        self.assertNotEqual(changedobj, obj)
        intervals = [
                (datetime(2000,4,13,hour=8), timedelta(minutes=50)),
                (datetime(1984,4,13,hour=9), timedelta(minutes=50)),
                (datetime(1984,4,14,hour=8), timedelta(minutes=50)),
                (datetime(1984,4,14,hour=9), timedelta(minutes=50))]
        changedobj = data.TimeDomain('Time', intervals, False)
        self.assertNotEqual(changedobj, obj)
        changedobj = 'ddd'
        self.assertNotEqual(changedobj, obj)


class TestTmatrixTimeDomain(unittest.TestCase):

    def test(self):
        obj1 = data.TmatrixTimeDomain('T', [1, 2])
        verifyObject(interfaces.ITmatrixTimeDomain, obj1)

    def testEquality(self):
        obj1 = data.TmatrixTimeDomain('T', [1, 2])
        obj2 = data.TmatrixTimeDomain('T', [1, 2])
        obj3 = data.TmatrixTimeDomain('T2', [3, 4])
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestDomainsvar(unittest.TestCase):
    """Test the Domainsvar data object."""

    def test(self):
        domainsvar = data.Domainsvar(u'Domain',
                ( 'Locations', u'Teachers', 'T' ))
        verifyObject(interfaces.IDomainsvar, domainsvar)
        verifyObject(interfaces.ICreateDomainsvar, domainsvar)
        self.assertEqual(domainsvar.getDomainsvarName(), 'Domain')
        self.assertEqual(domainsvar.getDomains(),
                ( 'Locations', u'Teachers', 'T' ))

    def testEquality(self):
        obj = data.Domainsvar('Domain',
                ( 'Locations', 'Teachers', 'T' ))
        changedobj = data.Domainsvar('Domain',
                ( 'Locations', 'Teachers', 'T' ))
        self.assertEqual(changedobj, obj)
        changedobj = data.Domainsvar('xxx',
                ( 'Locations', 'Teachers', 'T' ))
        self.assertNotEqual(changedobj, obj)
        changedobj = data.Domainsvar('Domain',
                ( 'Locations', 'Teachers', 'ccc' ))
        self.assertNotEqual(changedobj, obj)
        changedobj = 'ddd'
        self.assertNotEqual(changedobj, obj)


class TestClassifier(unittest.TestCase):

    def testEquality(self):
        obj1 = data.Classifier('classifier1', ('A', 'B'))
        obj2 = data.Classifier('classifier1', ('D', 'E'))
        self.assertNotEqual(obj1, obj2)
        self.assertNotEqual(obj1, 's')
        verifyObject(interfaces.IClassifier, obj1)
        verifyObject(interfaces.ICreateClassifier, obj1)


class TestConstraintFunction(unittest.TestCase):
    """Test the Constraint Function data object."""

    def test(self):
        constraint1 = data.ConstraintFunction([1, 2])
        constraint2 = data.ConstraintFunction([1, 2], ishard=False)
        self.assertRaises(TypeError, data.ConstraintFunction, [1, 2],
                ishard='other')
        self.assertEqual(constraint1.isHard(), True)
        self.assertEqual(constraint2.isHard(), False)
        verifyObject(interfaces.ICreateConstraintFunction, constraint1)

    def testEquality(self):
        constraint1 = data.ConstraintFunction([1, 2])
        constraint2 = data.ConstraintFunction([1, 2])
        constraint3 = data.ConstraintFunction([1, 2], ishard=False)
        constraint4 = data.ConstraintFunction([1, 5])
        self.assertEquals(constraint1, constraint2)
        self.assertNotEquals(constraint1, constraint3)
        self.assertNotEquals(constraint1, constraint4)

class TestFunction(unittest.TestCase):
    """Test the Function data object."""

    def test(self):
        arguments = ['1', 2, 'three']
        attributes = { 'attr1': 'atr1' }
        constraint = data.Function(arguments,
                attributes=attributes, name='Form 1 - Science in A100')
        self.assertEqual(constraint.getName(), 'Form 1 - Science in A100')
        self.assertEqual(constraint.getArguments(), arguments)
        self.assertEqual(constraint.getAttributes(), attributes)
        constraint = data.Function(arguments)
        self.assertEqual(constraint.getName(), None)
        self.assertEqual(constraint.getAttributes(), {})
        verifyObject(interfaces.IFunction, constraint)

    def testEquality(self):
        arguments = ['1', 2, 'three']
        attributes = {'attr1': 'atr1'}
        name = 'Form 1 - Science in A100'
        obj = data.Function(arguments, attributes=attributes, name=name)
        # Now change the obj in small ways and make sure it results in not =
        #obj name
        self.assertNotEqual(data.Function(arguments,
                attributes=attributes, name=name + 'c'), obj)
        #different arguments
        self.assertNotEqual(data.Function(['1', 2, '3'],
                attributes=attributes, name=name), obj)
        #different attributes
        self.assertNotEqual(data.Function(arguments,
                attributes={'foo': 'bar'}, name=name), obj)
        #missing name
        self.assertNotEqual(data.Function(arguments, attributes=attributes), obj)
        #missing attributes
        self.assertNotEqual(data.Function(arguments, name=name), obj)
        #different obj
        self.assertNotEqual(obj, 's')

class TestNotSameConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.NotSameConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.INotSameConstraint, obj1)

    def testEquality(self):
        obj1 = data.NotSameConstraint('const1', ('A', 'B'))
        obj2 = data.NotSameConstraint('const1', ('A', 'B'))
        obj3 = data.NotSameConstraint('const3', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestNoOverlapConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.NoOverlapConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.INoOverlapConstraint, obj1)

    def testEquality(self):
        obj1 = data.NoOverlapConstraint('const1', ('A', 'B'))
        obj2 = data.NoOverlapConstraint('const1', ('A', 'B'))
        obj3 = data.NoOverlapConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestPresetConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.PresetConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IPresetConstraint, obj1)

    def testEquality(self):
        obj1 = data.PresetConstraint('const1', ('A', 'B'))
        obj2 = data.PresetConstraint('const1', ('A', 'B'))
        obj3 = data.PresetConstraint('const', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestExcludeConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.ExcludeConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IExcludeConstraint, obj1)

    def testEquality(self):
        obj1 = data.ExcludeConstraint('const1', ('A', 'B'))
        obj2 = data.ExcludeConstraint('const1', ('A', 'B'))
        obj3 = data.ExcludeConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestOrderingConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.OrderingConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IOrderingConstraint, obj1)

    def testEquality(self):
        obj1 = data.OrderingConstraint('const1', ('A', 'B'))
        obj2 = data.OrderingConstraint('const1', ('A', 'B'))
        obj3 = data.OrderingConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestEventSprConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.EventSprConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IEventSprConstraint, obj1)

    def testEquality(self):
        obj1 = data.EventSprConstraint('const1', ('A', 'B'))
        obj2 = data.EventSprConstraint('const1', ('A', 'B'))
        obj3 = data.EventSprConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestFullSprConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.FullSprConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IFullSprConstraint, obj1)

    def testEquality(self):
        obj1 = data.FullSprConstraint('const1', ('A', 'B'))
        obj2 = data.FullSprConstraint('const1', ('A', 'B'))
        obj3 = data.FullSprConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestFreeSprConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.FreeSprConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IFreeSprConstraint, obj1)

    def testEquality(self):
        obj1 = data.FreeSprConstraint('const1', ('A', 'B'))
        obj2 = data.FreeSprConstraint('const1', ('A', 'B'))
        obj3 = data.FreeSprConstraint('const3', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestAttrCompConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.AttrCompConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IAttrCompConstraint, obj1)

    def testEquality(self):
        obj1 = data.AttrCompConstraint('const1', ('A', 'B'))
        obj2 = data.AttrCompConstraint('const1', ('A', 'B'))
        obj3 = data.AttrCompConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestResNoClashConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.ResNoClashConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IResNoClashConstraint, obj1)

    def testEquality(self):
        obj1 = data.ResNoClashConstraint('const1', ('A', 'B'))
        obj2 = data.ResNoClashConstraint('const1', ('A', 'B'))
        obj3 = data.ResNoClashConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestChksumConstraint(unittest.TestCase):

    def test(self):
        obj1 = data.ChksumConstraint('const1', ('A', 'B'))
        verifyObject(interfaces.IChksumConstraint, obj1)

    def testEquality(self):
        obj1 = data.ChksumConstraint('const1', ('A', 'B'))
        obj2 = data.ChksumConstraint('const1', ('A', 'B'))
        obj3 = data.ChksumConstraint('const2', ('A', 'B'))
        self.assertEqual(obj1, obj2)
        self.assertNotEqual(obj1, obj3)
        self.assertNotEqual(obj1, 's')


class TestResult(unittest.TestCase):

    def test(self):
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        desc = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        best = [['a', 'b'], ['c', 'd']]
        obj1 = data.Result(desc, best)
        self.assertEquals(desc, obj1.getDescription())
        self.assertEquals(best, obj1.getBestResult())
        verifyObject(interfaces.IResult, obj1)
        verifyObject(interfaces.ICreateResult, obj1)

    def testEquality(self):
        descriptive_authors = ['C. Bukowski', 'P.J. Gutierez']
        descriptive_desc = 'The same minds.'
        descriptive_refs = [u'Ham on Rye', 'Anclado en la tierra de nadie']
        desc = data.Description(descriptive_authors, descriptive_desc,
                descriptive_refs)
        best = [['a', 'b'], ['c', 'd']]
        obj1 = data.Result(desc, best)
        obj2 = data.Result(desc, best)
        self.assertEqual(obj1, obj2)
        desc2 = data.Description(descriptive_authors, descriptive_desc + 'c',
                descriptive_refs)
        obj3 = data.Result(desc2, best)
        self.assertNotEqual(obj1, obj3)
        best2 = [['b', 'a'], ['d', 'c']]
        obj4 = data.Result(desc, best2)
        self.assertNotEqual(obj1, obj4)
        self.assertNotEqual(obj1, 's')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestDocInfo))
    suite.addTest(unittest.makeSuite(TestDescription))
    suite.addTest(unittest.makeSuite(TestInputDescription))
    suite.addTest(unittest.makeSuite(TestTTMLSet))
    suite.addTest(unittest.makeSuite(TestAttributedSet))
    suite.addTest(unittest.makeSuite(TestVariableSet))
    suite.addTest(unittest.makeSuite(TestDomain))
    suite.addTest(unittest.makeSuite(TestTimeDomain))
    suite.addTest(unittest.makeSuite(TestTmatrixTimeDomain))
    suite.addTest(unittest.makeSuite(TestDomainsvar))
    suite.addTest(unittest.makeSuite(TestClassifier))
    suite.addTest(unittest.makeSuite(TestFunction))
    suite.addTest(unittest.makeSuite(TestConstraintFunction))
    suite.addTest(unittest.makeSuite(TestNotSameConstraint))
    suite.addTest(unittest.makeSuite(TestNoOverlapConstraint))
    suite.addTest(unittest.makeSuite(TestPresetConstraint))
    suite.addTest(unittest.makeSuite(TestExcludeConstraint))
    suite.addTest(unittest.makeSuite(TestOrderingConstraint))
    suite.addTest(unittest.makeSuite(TestEventSprConstraint))
    suite.addTest(unittest.makeSuite(TestFullSprConstraint))
    suite.addTest(unittest.makeSuite(TestFreeSprConstraint))
    suite.addTest(unittest.makeSuite(TestAttrCompConstraint))
    suite.addTest(unittest.makeSuite(TestResNoClashConstraint))
    suite.addTest(unittest.makeSuite(TestChksumConstraint))
    suite.addTest(unittest.makeSuite(TestResult))
    return suite
